@@ -10,10 +10,11 @@ from pathlib import Path
 import torch
 import torch.nn.functional as F
 from torch import nn, einsum
+from torch.special import expm1
 import torchvision.transforms as T
 
 from einops import rearrange, repeat
-from einops.layers.torch import Rearrange
+from einops.layers.torch import Rearrange, Reduce
 from einops_exts import rearrange_many, repeat_many, check_shape
 from einops_exts.torch import EinopsToAndFrom
 
@@ -82,6 +83,22 @@ def log(t, eps = 1e-12):
 def l2norm(t):
     return F.normalize(t, dim = -1)
 
+def right_pad_dims_to(x, t):
+    padding_dims = x.ndim - t.ndim
+    if padding_dims <= 0:
+        return t
+    return t.view(*t.shape, *((1,) * padding_dims))
+
+def masked_mean(t, *, dim, mask = None):
+    if not exists(mask):
+        return t.mean(dim = dim)
+
+    denom = mask.sum(dim = dim, keepdim = True)
+    mask = rearrange(mask, 'b n -> b n 1')
+    masked_t = t.masked_fill(~mask, 0.)
+
+    return masked_t.sum(dim = dim) / denom.clamp(min = 1e-5)
+
 def resize_image_to(image, target_image_size):
     orig_image_size = image.shape[-1]
 
@@ -216,7 +233,7 @@ class GaussianDiffusion(nn.Module):
 
         # below: log calculation clipped because the posterior variance is 0 at the beginning of the diffusion chain
 
-        register_buffer('posterior_log_variance_clipped', torch.log(posterior_variance.clamp(min =1e-20)))
+        register_buffer('posterior_log_variance_clipped', log(posterior_variance, eps = 1e-20))
         register_buffer('posterior_mean_coef1', betas * torch.sqrt(alphas_cumprod_prev) / (1. - alphas_cumprod))
         register_buffer('posterior_mean_coef2', (1. - alphas_cumprod_prev) * torch.sqrt(alphas) / (1. - alphas_cumprod))
 
@@ -262,10 +279,28 @@ class GaussianDiffusion(nn.Module):
             extract(self.sqrt_recipm1_alphas_cumprod, t, x_t.shape) * noise
         )
 
+# gaussian diffusion with continuos time helper functions and classes
+# large part of this was thanks to @crowsonkb at https://github.com/crowsonkb/v-diffusion-jax/blob/master/diffusion/utils.py
+
+@torch.jit.script
+def beta_linear_log_snr(t):
+    return -torch.log(expm1(1e-4 + 10 * (t ** 2)))
+
+def alpha_cosine_log_snr(t):
+    raise NotImplementedError
+
+def log_snr_to_alpha_sigma(log_snr):
+    return torch.sqrt(torch.sigmoid(log_snr)), torch.sqrt(torch.sigmoid(-log_snr))
+
 class GaussianDiffusionContinuousTimes(GaussianDiffusion):
-    def __init__(self, *, beta_schedule, timesteps):
+    def __init__(self, *, beta_schedule, **kwargs):
         super().__init__()
-        raise NotImplementedError
+        if beta_schedule == 'linear':
+            self.log_snr = beta_linear_log_snr
+        elif beta_schedule == "cosine":
+            raise NotImplementedError
+        else:
+            raise ValueError(f'invalid noise schedule {beta_schedule}')
 
     def get_times(self, batch_size, noise_level):
         device = self.betas.device
@@ -278,14 +313,35 @@ class GaussianDiffusionContinuousTimes(GaussianDiffusion):
     def get_learned_posterior_log_variance(self, var_interp_frac_unnormalized, x_t, t):
         raise NotImplementedError
 
-    def q_posterior(self, x_start, x_t, t):
-        raise NotImplementedError
+    def q_posterior(self, x_start, x_t, t, t_next):
+        """ https://openreview.net/attachment?id=2LdBqxc1Yv&name=supplementary_material """
+        """ µ still has to be done (eq. 32)"""
+        log_snr = self.log_snr(t)
+        log_snr_next = self.log_snr(t_next)
+        log_snr, log_snr_next = map(partial(right_pad_dims_to, x_t), (log_snr, log_snr_next))
 
-    def q_sample(self, x_start, t, noise=None):
-        raise NotImplementedError
+        alpha, sigma = log_snr_to_alpha_sigma(log_snr)
+        alpha_next, sigma_next = log_snr_to_alpha_sigma(log_snr_next)
+
+        posterior_mean = None # todo, eq 32 from above, but done in a way to receive the x_start from predict_start_from_noise, since the x_start needs to be clipped (by dynamic thresholding?)
+
+        # following (eq. 33)
+        posterior_variance = (sigma_next ** 2) * -expm1(log_snr_next - log_snr)
+        posterior_log_variance_clipped = log(posterior_variance, eps = 1e-20)
+        return posterior_mean, posterior_variance, posterior_log_variance_clipped
+
+    def q_sample(self, x_start, t, noise = None):
+        noise = default(noise, lambda: torch.randn_like(x_start))
+        log_snr = self.log_snr(x_start, t)
+        log_snr = right_pad_dims_to(x_start, log_snr)
+        alpha, sigma =  log_snr_to_alpha_sigma(log_snr)
+        return alpha * x_start + sigma * noise
 
     def predict_start_from_noise(self, x_t, t, noise):
-        raise NotImplementedError
+        log_snr = self.log_snr(t)
+        log_snr = right_pad_dims_to(x_t, log_snr)
+        alpha, sigma = log_snr_to_alpha_sigma(log_snr)
+        return (x_t - sigma * noise) / alpha.clamp(min = 1e-5)
 
 # norms and residuals
 
@@ -383,13 +439,24 @@ class PerceiverResampler(nn.Module):
         dim_head = 64,
         heads = 8,
         num_latents = 64,
+        num_latents_mean_pooled = 4, # number of latents derived from mean pooled representation of the sequence
         max_seq_len = 512,
-        ff_mult = 4
+        ff_mult = 4,
     ):
         super().__init__()
-        self.latents = nn.Parameter(torch.randn(num_latents, dim))
         self.pos_emb = nn.Embedding(max_seq_len, dim)
 
+        self.latents = nn.Parameter(torch.randn(num_latents, dim))
+
+        self.to_latents_from_mean_pooled_seq = None
+
+        if num_latents_mean_pooled > 0:
+            self.to_latents_from_mean_pooled_seq = nn.Sequential(
+                LayerNorm(dim),
+                nn.Linear(dim, dim * num_latents_mean_pooled),
+                Rearrange('b (n d) -> b n d', n = num_latents_mean_pooled)
+            )
+
         self.layers = nn.ModuleList([])
         for _ in range(depth):
             self.layers.append(nn.ModuleList([
@@ -401,12 +468,17 @@ class PerceiverResampler(nn.Module):
         n, device = x.shape[1], x.device
         pos_emb = self.pos_emb(torch.arange(n, device = device))
 
-        x = x + pos_emb
+        x_with_pos = x + pos_emb
 
         latents = repeat(self.latents, 'n d -> b n d', b = x.shape[0])
 
+        if exists(self.to_latents_from_mean_pooled_seq):
+            meanpooled_seq = masked_mean(x, dim = 1, mask = torch.ones(x.shape[:2], device = x.device, dtype = torch.bool))
+            meanpooled_latents = self.to_latents_from_mean_pooled_seq(meanpooled_seq)
+            latents = torch.cat((meanpooled_latents, latents), dim = -2)
+
         for attn, ff in self.layers:
-            latents = attn(x, latents, mask = mask) + latents
+            latents = attn(x_with_pos, latents, mask = mask) + latents
             latents = ff(latents) + latents
 
         return latents
@@ -752,8 +824,8 @@ class CrossEmbedLayer(nn.Module):
 class Unet(nn.Module):
     def __init__(
         self,
-        dim,
         *,
+        dim,
         image_embed_dim = 1024,
         text_embed_dim = get_encoded_dim(DEFAULT_T5_NAME),
         num_resnet_blocks = 1,
@@ -780,15 +852,14 @@ class Unet(nn.Module):
         cross_embed_downsample = False,
         cross_embed_downsample_kernel_sizes = (2, 4),
         attn_pool_text = True,
-        attn_pool_num_latents = 32,
-        **kwargs
+        attn_pool_num_latents = 32
     ):
         super().__init__()
         # save locals to take care of some hyperparameters for cascading DDPM
 
         self._locals = locals()
-        del self._locals['self']
-        del self._locals['__class__']
+        self._locals.pop('self', None)
+        self._locals.pop('__class__', None)
 
         # for eventual cascading diffusion
 
@@ -985,7 +1056,7 @@ class Unet(nn.Module):
         # add low resolution conditioning, if present
 
         assert not (self.lowres_cond and not exists(lowres_cond_img)), 'low resolution conditioning image must be present'
-        assert not (self.lowres_cond and not exists(lowres_noise_times)), 'low resolution conditoining noise time must be present'
+        assert not (self.lowres_cond and not exists(lowres_noise_times)), 'low resolution conditioning noise time must be present'
 
         if exists(lowres_cond_img):
             x = torch.cat((x, lowres_cond_img), dim = 1)
